'use strict';

var web = require('solid-js/web');
var solidJs = require('solid-js');

const HIDDEN_TEXTAREA_STYLE = {
  "min-height": "0",
  "max-height": "none",
  height: "0",
  visibility: "hidden",
  overflow: "hidden",
  position: "absolute",
  "z-index": "-1000",
  top: "0",
  right: "0"
};
const forceHiddenStyles = node => {
  Object.keys(HIDDEN_TEXTAREA_STYLE).forEach(key => {
    node.style.setProperty(key, HIDDEN_TEXTAREA_STYLE[key], "important");
  });
};
var forceHiddenStyles$1 = forceHiddenStyles;

let hiddenTextarea = null;
const getHeight = (node, sizingData) => {
  const height = node.scrollHeight;
  if (sizingData.sizingStyle["box-sizing"] === "border-box") {
    // border-box: add border, since height = content + padding + border
    return height + sizingData.borderSize;
  }

  // remove padding, since height = content
  return height - sizingData.paddingSize;
};
function calculateNodeHeight(sizingData, value, minRows = 1, maxRows = Infinity) {
  if (!hiddenTextarea) {
    hiddenTextarea = document.createElement("textarea");
    hiddenTextarea.setAttribute("tabindex", "-1");
    hiddenTextarea.setAttribute("aria-hidden", "true");
    forceHiddenStyles$1(hiddenTextarea);
  }
  if (hiddenTextarea.parentNode === null) {
    document.body.appendChild(hiddenTextarea);
  }
  const {
    paddingSize,
    borderSize,
    sizingStyle
  } = sizingData;
  const {
    "box-sizing": boxSizing
  } = sizingStyle;
  Object.keys(sizingStyle).forEach(_key => {
    const key = _key;
    // TODO:  style[key as any] is needed since .style follows react's camelCase CSSStyleDeclaration
    hiddenTextarea.style[key] = sizingStyle[key];
  });
  forceHiddenStyles$1(hiddenTextarea);
  hiddenTextarea.value = value;
  let height = getHeight(hiddenTextarea, sizingData);

  // measure height of a textarea with a single row
  hiddenTextarea.value = "x";
  const rowHeight = hiddenTextarea.scrollHeight - paddingSize;
  let minHeight = rowHeight * minRows;
  if (boxSizing === "border-box") {
    minHeight = minHeight + paddingSize + borderSize;
  }
  height = Math.max(minHeight, height);
  let maxHeight = rowHeight * maxRows;
  if (boxSizing === "border-box") {
    maxHeight = maxHeight + paddingSize + borderSize;
  }
  height = Math.min(maxHeight, height);
  return [height, rowHeight];
}

const pick = (props, obj) => props.reduce((acc, prop) => {
  acc[prop] = obj[prop];
  return acc;
}, {});

const SIZING_STYLE = ["border-bottom-width", "border-left-width", "border-right-width", "border-top-width", "box-sizing", "font-family", "font-size", "font-style", "font-zeight", "letter-spacing", "line-height", "padding-bottom", "padding-left", "padding-right", "padding-top",
// non-standard
"tab-size", "text-indent",
// non-standard
"text-rendering", "text-transform", "width", "word-break"];

// TODO:  solid afaik does not have an equivalent to CSSStyleDeclaration
//        replace this line once there is a proper solid equivalent

const isIE = typeof document !== "undefined" ? !!document.documentElement.currentStyle : false;
const getSizingData = node => {
  const style = window.getComputedStyle(node);
  if (!style) {
    return null;
  }
  const sizingStyle = pick(SIZING_STYLE, style);
  const {
    "box-sizing": boxSizing
  } = sizingStyle;

  // probably node is detached from DOM, can't read computed dimensions
  if (!boxSizing) {
    return null;
  }

  // IE (Edge has already correct behaviour) returns content width as computed width
  // so we need to add manually padding and border widths
  if (isIE && boxSizing === "border-box") {
    sizingStyle.width = parseFloat(sizingStyle.width) + parseFloat(sizingStyle["border-right-width"]) + parseFloat(sizingStyle["border-left-width"]) + parseFloat(sizingStyle["padding-right"]) + parseFloat(sizingStyle["padding-left"]) + "px";
  }
  const paddingSize = parseFloat(sizingStyle["padding-bottom"]) + parseFloat(sizingStyle["padding-top"]);
  const borderSize = parseFloat(sizingStyle["border-bottom-width"]) + parseFloat(sizingStyle["border-top-width"]);
  return {
    sizingStyle,
    paddingSize,
    borderSize
  };
};
var getSizingData$1 = getSizingData;

const useWindowResizeListener = listener => {
  const handler = event => listener(event);
  window.addEventListener("resize", handler);
  solidJs.onCleanup(() => window.removeEventListener("resize", handler));
};

const _tmpl$ = /*#__PURE__*/web.template(`<textarea></textarea>`, 2);
function TextareaAutosize(props) {
  const [textarea, setTextarea] = solidJs.createSignal();
  let heightRef = 0;
  let measurementsCacheRef = undefined;
  const resizeTextarea = () => {
    const node = textarea();
    if (!node) return;
    const nodeSizingData = props.cacheMeasurements && measurementsCacheRef ? measurementsCacheRef : getSizingData$1(node);
    if (!nodeSizingData) {
      return;
    }
    measurementsCacheRef = nodeSizingData;
    const [height, rowHeight] = calculateNodeHeight(nodeSizingData, node.value || node.placeholder || "x", props.minRows, props.maxRows);
    if (heightRef !== height) {
      heightRef = height;
      node.style.setProperty("height", `${height}px`, "important");
      props.onHeightChange?.(height, {
        rowHeight
      });
    }
  };
  const handleChange = event => {
    resizeTextarea();
    if (typeof props.oninput === "function") props.oninput(event);
    if (typeof props.onInput === "function") props.onInput(event);
  };
  solidJs.createEffect(solidJs.on(() => props.value, resizeTextarea));
  solidJs.createEffect(() => {
    if (typeof document !== "undefined" && textarea()) {
      resizeTextarea();
      useWindowResizeListener(resizeTextarea);
    }
  });
  return (() => {
    const _el$ = _tmpl$.cloneNode(true);
    web.use(element => setTextarea(element), _el$);
    web.spread(_el$, web.mergeProps(props, {
      "oninput": handleChange,
      "onInput": handleChange
    }), false, false);
    return _el$;
  })();
}

module.exports = TextareaAutosize;
//# sourceMappingURL=index.js.map
